/***************************************************************************
 *            audioEvent.h
 *
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef AUDIOEVENT_H
#define AUDIOEVENT_H

#include "../base/instrument.h"
#include <math.h>
#include <iostream>

class audioEvent			//audio event triggered by midi
{
	public:
			audioEvent(variation*,instrument*,unsigned long,double,int,double,double,double,bool);
			inline variation* getVariation(){return var;};
			inline instrument* getInstrument(){return instru;};
			inline double getPitch(){return pitch;};
			inline double getPanLeft(){return panLeft;};
			inline double getPanRight(){return panRight;};
			inline unsigned long getID(){return id;};
			inline bool getEvent(){return event;};
			inline int getOffset(){return (int)offset;};
			inline int getJackStereoChannel(){return jackStereoChannel;};
			//we add the pitch to the offset, if the play mode ids repeat, we go back to the beginning of the sample
			inline void addOffset(){offset+=pitch;if(playMode==2){if(offset>=var->getSample()->getFramesCount())offset-=var->getSample()->getFramesCount();}}; 
			//we start fading out
			inline void fadeOut(int nframes)
			{
				fade=volume/1000.0;
			};
			inline int getPlayingLength(int nframes) 
			{
				if(playMode==2) //repeatMode -> always return nframes
					return (int)nframes;
				else
				{
					int length = (floor( ((double)(var->getSample()->getFramesCount()) - offset)/pitch) < nframes )? (int)floor( ((double)(var->getSample()->getFramesCount()) - offset)/pitch) : (int)nframes;
					return length;
				}
			};
			inline double getVolume()
			{
				//if we are fading out, we decrease the volume
				if(fade!=1.0)
					volume-=fade;
				return volume;
			};
			
	private:
			variation* var;
			instrument* instru;
			double pitch;
			unsigned long id; 
			bool event;
			double volume;
			double panLeft;
			double panRight;
			double fade;
			int jackStereoChannel;
			double offset;
			int playMode;
};

#endif
